PrimeCurveParams

Trait PrimeCurveParams 

pub trait PrimeCurveParams:
    Curve<FieldBytesSize: ModulusSize>
    + CurveArithmetic<AffinePoint = AffinePoint<Self>, ProjectivePoint = ProjectivePoint<Self>, Scalar: PrimeFieldExt + WnafSize>
    + FieldArithmetic<FieldElement: PrimeFieldExt>
    + PrimeCurve {
    type PointArithmetic: PointArithmetic<Self>;
    type Backend: MulBackend<Self>;

    const EQUATION_A: Self::FieldElement;
    const EQUATION_B: Self::FieldElement;
    const GENERATOR: (Self::FieldElement, Self::FieldElement);
}
Parameters for elliptic curves of prime order which can be described by the short Weierstrass equation.

Required Associated Constants

const EQUATION_A: Self::FieldElement

Coefficient a in the curve equation.

const EQUATION_B: Self::FieldElement

Coefficient b in the curve equation.

const GENERATOR: (Self::FieldElement, Self::FieldElement)

Generator point’s affine coordinates: (x, y).

Required Associated Types

type PointArithmetic: PointArithmetic<Self>

Point arithmetic implementation, might be optimized for this specific curve

type Backend: MulBackend<Self>

Scalar arithmetic backend implementation.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors